import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {action, computed, override} from "mobx";
import {ScannerRuleModel} from "./ScannerRuleStore";
import {Moment} from "moment/moment";

export type SCAN_TRIGGER = 'trigger' | 'continual';
export type SCANNER_TYPE = 'DatalogicMatrix120' | 'WenglorB50' | 'SickClv62x'

@model({
    transportLayer: '/api/scanners',
    permissionTarget: "Scanner",
    routingInfo: {
        create: '/scanners/create',
        bulkCreate: '/scanners/bulk_create',
        read: '/scanners/:id/edit',
        list: '/scanners',
    }
})

export class ScannerModel extends Model {
    constructor() {
        super();
        super.setup();
    }

    @attribute({
        label: t`Name`,
        validation: yup.string().required(t`Scanner name is a required field`)
    })
    name: string = '';

    @attribute({
        label: t`Scanner Type`,
        validation: yup.string().required(t`Scanner Type is a required field`)
    })
    scanner_type: SCANNER_TYPE | null = null;

    @attribute({
        label: t`Scan Trigger`,
        validation: yup.string().required(t`Scanner Trigger is a required field`)
    })
    scan_trigger: SCAN_TRIGGER | null = 'trigger';

    @attribute({
        label: t`IP Address`,
        validation: yup.string().matches(/^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$/, t`Must be an ip address`)
    })
    ip_address: string = '';

    @attribute({
        label: t`Last Read Value`,
        validation: yup.string().nullable()
    })
    last_read_value: string = '';

    @attribute({
        label: t`Last Read Time`,
        validation: yup.string().nullable()
    })
    last_read_time: Moment | null = null;

    @attribute({})
    parameters: ScannerParams | null = null;

    @attribute({})
    rules_to_add: ScannerRuleModel[] = []

    @attribute({
        label: t`Minimum Scan Length`,
        validation: yup.number().min(1)
    })
    min_scan_length: number = 3;

    @attribute() // generally only used for continual scanners
    online: boolean = false;

    @computed
    get requiresParameters(): boolean {
        return this.scan_trigger === "continual";
    }

    @override
    handleSelectChange(name: string | undefined, value: (string | Array<string> | boolean | Array<boolean> | number | Array<number>)) {
        super.handleSelectChange(name, value);

        if(name === "scan_trigger"){
            this.nestedAssignment("scanner_type",null);

            if(this.viewModel.scan_trigger === "continual"){
                this.nestedAssignment("parameters","{}");
                this.ensureScannerParameters();
            }
        }

        if(name === "scanner_type"){
            if(this.viewModel.scanner_type === null){
                this.nestedAssignment("parameters",null);
            }
        }

        if(name === "parameters.eticket_actions.action"){
            if(this.viewModel.parameters && this.viewModel.parameters["eticket_actions"]["action"] !== "template_change"){
                this.nestedAssignment("parameters.eticket_actions.template_name",null);
            }
        }
    }

    @override
    updateFromJson(json: Model, isClone?: boolean) {
        super.updateFromJson(json, isClone);
        this.ensureScannerParameters();
    }

    @override
    resetView() {
        // If the view is refreshed, we need to manually re-parse parameters JSON
        super.resetView();
        this.ensureScannerParameters();
    }

    private ensureScannerParameters(): void {
        const ensureParameter = (setting: string, defaultValue: any, subSettings: {name: string, value: any}[] = []): void => {
            if(!this.viewModel.parameters){
                return;
            }

            if(this.viewModel.parameters[setting] === undefined){
                this.viewModel.parameters[setting] = defaultValue;
            }

            for(let subSetting of subSettings){
                if(this.viewModel.parameters[setting][subSetting.name] === undefined){
                    this.viewModel.parameters[setting][subSetting.name] = subSetting.value;
                }
            }
        }

        if(typeof this.viewModel.parameters === 'string'){
            this.viewModel.parameters = JSON.parse(this.viewModel.parameters) as ScannerParams;
        }

        if(this.viewModel.scan_trigger === "continual"){
            ensureParameter("auto_start_with_service", false);
            ensureParameter("value_separator", ";");
            ensureParameter("no_read_value", "NoRead");
            ensureParameter("eticket_actions", {}, [
                {name: "action", value: null},
                {name: "template_name", value: null},
            ]);
        }
    }

    @computed
    get sendETicketTemplateEnabled(): boolean {
        if(!this.viewModel.parameters || !this.viewModel.parameters.eticket_actions){
            return false;
        }
        return this.viewModel.parameters.eticket_actions['action'] == 'template_change';
    }


    @action
    async ping() {
        await this.store.transportLayer.fetch(`/${this.id}/ping_v2`);
    }

    @action
    async scan() {
        const response = await this.store.transportLayer.fetch(`/${this.id}/scan`);

        const data = await response.json();

        this.last_read_value = data.last_read_value;
        this.last_read_time = data.last_read_time;
    }

    @action
    async start() {
        await this.store.transportLayer.fetch(`/${this.id}/start`, {
            method: "PUT",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({})
        });
    }

    @action
    async stop() {
        await this.store.transportLayer.fetch(`/${this.id}/stop`, {
            method: "PUT",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({})
        });
    }

    @computed
    get last_scan_successful(): boolean {
        return this.last_read_value !== 'fail';
    }
}

type ScannerParams = {
    [key: string]: any;
}

export type ContinualScannerParameters = {
    auto_start_with_service: boolean;
    value_separator: string;
    no_read_value: string;
    eticket_actions: {
        action: null | 'template_change' | 'ticket_release';
        template_name: string | null;
    }

} & ScannerParams;

export const ScannerStore = ScannerModel.store as DataStore<ScannerModel>;
