module ScannerHandler
  module Continual
    class BaseContinualScannerHandler
      require "socket"

      class InvalidScannerParameters < StandardError; end

      def initialize
        @client = nil
        @thread = nil
      end

      def start(scanner)
        initialize_configuration(scanner)

        @thread = Thread.new do
          Rails.application.executor.wrap do
            open_connection
            scan_loop
            stop
          end
        end
      end

      def stop
        log('stopping')
        @client.close unless @client.nil?
      rescue => err
        log("failed to close socket: #{err}", 'error')
      ensure
        @client = nil
        @thread.kill unless @thread.nil?
        @thread = nil
        broadcast_status
      end

      def running?
        !@thread.nil?
      end

      private

      def initialize_configuration(scanner)
        @scanner = scanner
        ensure_parameters
      end

      def ensure_parameters
        raise InvalidScannerParameters if @scanner.resolved_parameters.blank?

        req_params = [:no_read_value, :value_separator]

        req_params.each do | param |
          raise InvalidScannerParameters if @scanner.resolved_parameters[param].nil?
        end
      end

      def open_connection
        begin
          log("opening connection")
          @client = TCPSocket.new(@scanner.ip_address, port)
          log("listening")
          broadcast_status
        rescue => err
          @client = nil
          log("failed to open socket: #{err}", 'error')
          stop
        end
      end

      def port
        raise NotImplementedError
      end

      def scan_loop
        loop do
          break unless connection_opened?

          scan_value = read

          next if scan_value == no_read_value
          next if scan_value == @scanner.last_read_value
          next if scan_value.length < @scanner.min_scan_length

          log("received scan value #{scan_value}")

          @scanner.update!({last_read_value: scan_value, last_read_time: Time.zone.now})

          process_actions
        end
      end


      def read
        raise NotImplementedError
      end

      def process_actions
        handle_eticket_action
      end

      def connection_opened?
        !@client.nil?
      end

      def handle_eticket_action
        return if @scanner.resolved_parameters[:eticket_actions].nil? || @scanner.resolved_parameters[:eticket_actions][:action].nil?

        if @scanner.resolved_parameters[:eticket_actions][:action] == 'template_change'
          send_template_change
        elsif @scanner.resolved_parameters[:eticket_actions][:action] == 'ticket_release'
          send_template_release
        end
      end

      def send_template_change
        raise InvalidScannerParameters if @scanner.resolved_parameters[:eticket_actions][:template_name].blank?

        SendToModuleJob.perform_later('rx_paperless', 'api/external/v1/orders/bulk-update', :put, {
          template: @scanner.resolved_parameters[:eticket_actions][:template_name],
          token: rx_paperless_token,
          orders: [
            {
              number: @scanner.last_read_value
            }
          ]
        })
      end

      def send_template_release
        SendToModuleJob.perform_later('rx_paperless', 'api/external/v1/orders/bulk-release', :put, {
          token: rx_paperless_token,
          orders: [
            {
              number: @scanner.last_read_value
            }
          ]
        })
      end

      def rx_paperless_token
        'rxmoduleinternal'
      end

      def value_separator
        @scanner.resolved_parameters[:value_separator]
      end

      def no_read_value
        @scanner.resolved_parameters[:no_read_value]
      end

      def broadcast_status
        ActionCable.server.broadcast "everything", {
          model: JSON.parse(@scanner.to_json),
          class: @scanner.class.name,
          action: :update
        }
      end

      def log(msg, level='info')
        Rails::logger.send(level, "[#{@scanner.name}] #{msg}")
      end
    end
  end
end
