class Scanner < ApplicationRecord
  has_many :scanner_rules

  validates :ip_address, presence: true
  validates :scanner_type, inclusion: {in: %w(DatalogicMatrix120 WenglorB50 SickClv62x)}
  validates :scan_trigger, inclusion: {in: %w(trigger continual)}

  enum scan_trigger: [:trigger, :continual]

  class ActionNotSupportByScannerType < StandardError; end

  concerning :TriggerScanner do
    def ensure_trigger_scanner
      raise ActionNotSupportByScannerType unless trigger?
    end

    def ping
      ensure_trigger_scanner

      handler = scan_handler
      handler.ping
    end

    def scan
      ensure_trigger_scanner

      handler = scan_handler

      response = handler.scan

      if response[:success]
        update!(last_read_time: Time.zone.now, last_read_value: apply_scan_rules( response[:value]))
      else
        if response[:scan_failed]
          update!(last_read_time: Time.zone.now, last_read_value: 'fail')
        end
      end

      response
    end
  end

  concerning :ContinualScanner do
    def ensure_continual_scanner
      raise ActionNotSupportByScannerType unless continual?
    end

    def start
      ensure_continual_scanner
      scan_handler.start(self)
    end

    def stop
      ensure_continual_scanner
      scan_handler.stop
    end

    def running?
      scan_handler.running?
    end
  end

  def add_rules(rule_params)
    rule_params.each do |rule_param|
      # add_to_target adds to the collection without saving.
      association(:scanner_rules).add_to_target(ScannerRule.new(rule_param))
    end
  end

  def apply_scan_rules(result)
    rules = scanner_rules.to_a

    return result if rules.size == 0

    rules.sort_by {|rule| rule.priority}
    rules.each do |rule|
      if rule.operation == 'delete'
        result.slice!(rule.start_position, rule.end_position)
      elsif rule.operation == 'append'
        result << rule.value
      elsif rule.operation == 'prepend'
        result.prepend(rule.value)
      end
    end
    result
  end

  def to_json(*args)
    render_jbuilder_partial
  end

  def resolved_parameters
    @resolved_parameters ||= JSON.parse(parameters, symbolize_names: true)
  end

  private

  def scan_handler
    ScannerManager.handler_for_model(self)
  end
end